@@ -3,7 +3,7 @@
  * 
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-6 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-8 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 1999-2000 Michael Cornwell <cornwell@acm.org>
  * Copyright (C) 2000 Andre Hedrick <andre@linux-ide.org>
  *
@@ -36,7 +36,7 @@
 #include "extern.h"
 #include "utility.h"
 
-const char *atacmds_c_cvsid="$Id: atacmds.cpp,v 1.177 2006/10/27 21:30:02 chrfranke Exp $"
+const char *atacmds_c_cvsid="$Id: atacmds.cpp,v 1.190 2008/03/04 22:09:47 ballen4705 Exp $"
 ATACMDS_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSIATA_H_CVSID UTILITY_H_CVSID;
 
 // to hold onto exit code for atexit routine
@@ -514,6 +514,25 @@ void swap8(char *location){
   return;
 }
 
+// Invalidate serial number and adjust checksum in IDENTIFY data
+static void invalidate_serno(ata_identify_device * id){
+  unsigned char sum = 0;
+  for (unsigned i = 0; i < sizeof(id->serial_no); i++) {
+    sum += id->serial_no[i]; sum -= id->serial_no[i] = 'X';
+  }
+#ifndef __NetBSD__
+  bool must_swap = !!isbigendian();
+  if (must_swap)
+    swapx(id->words088_255+255-88);
+#endif
+  if ((id->words088_255[255-88] & 0x00ff) == 0x00a5)
+    id->words088_255[255-88] += sum << 8;
+#ifndef __NetBSD__
+  if (must_swap)
+    swapx(id->words088_255+255-88);
+#endif
+}
+
 static char *commandstrings[]={
   "SMART ENABLE",
   "SMART DISABLE",
@@ -532,18 +551,20 @@ static char *commandstrings[]={
   "WARNING (UNDEFINED COMMAND -- CONTACT DEVELOPERS AT " PACKAGE_BUGREPORT ")\n"
 };
 
-void prettyprint(unsigned char *stuff, char *name){
-  int i,j;
+static void prettyprint(const unsigned char *p, const char *name){
   pout("\n===== [%s] DATA START (BASE-16) =====\n", name);
-  for (i=0; i<32; i++){
-    pout("%03d-%03d: ", 16*i, 16*(i+1)-1);
-    for (j=0; j<15; j++)
-      pout("%02x ",*stuff++);
-    pout("%02x\n",*stuff++);
-  }
+  for (int i=0; i<512; i+=16, p+=16)
+    // print complete line to avoid slow tty output and extra lines in syslog.
+    pout("%03d-%03d: %02x %02x %02x %02x %02x %02x %02x %02x "
+                    "%02x %02x %02x %02x %02x %02x %02x %02x\n",
+         i, i+16-1,
+         p[ 0], p[ 1], p[ 2], p[ 3], p[ 4], p[ 5], p[ 6], p[ 7],
+         p[ 8], p[ 9], p[10], p[11], p[12], p[13], p[14], p[15]);
   pout("===== [%s] DATA END (512 Bytes) =====\n\n", name);
 }
 
+static int parsedev_command_interface(int fd, smart_command_set command, int select, char * data);
+
 // This function provides the pretty-print reporting for SMART
 // commands: it implements the various -r "reporting" options for ATA
 // ioctls.
@@ -592,13 +613,9 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
   }
 
 
-  // If reporting is enabled, say what input was sent to the command
-  if (con->reportataioctl && sendsdata){
-    pout("REPORT-IOCTL: DeviceFD=%d Command=%s", device, commandstrings[command]);
-    // if requested, pretty-print the output data structure
-    if (con->reportataioctl>1)
-      prettyprint((unsigned char *)data, commandstrings[command]);
-  }
+  // if requested, pretty-print the input data structure
+  if (con->reportataioctl>1 && sendsdata)
+    prettyprint((unsigned char *)data, commandstrings[command]);
 
   // In case the command produces an error, we'll want to know what it is:
   errno=0;
@@ -621,10 +638,17 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
   case CONTROLLER_HPT:
     retval=highpoint_command_interface(device, command, select, data);
     break;
+  case CONTROLLER_PARSEDEV:
+    retval=parsedev_command_interface(device, command, select, data);
+    break;
   default:
     retval=ata_command_interface(device, command, select, data);
   }
 
+  // If requested, invalidate serial number before any printing is done
+  if ((command == IDENTIFY || command == PIDENTIFY) && !retval && con->dont_print_serial)
+    invalidate_serno((ata_identify_device *)data);
+
   // If reporting is enabled, say what output was produced by the command
   if (con->reportataioctl){
     if (errno)
@@ -700,7 +724,7 @@ int ataReadHDIdentity (int device, struct ata_identify_device *buf){
 
 #ifndef __NetBSD__
   // if machine is big-endian, swap byte order as needed
-  // (the NetBSD kernel does deliver the results in host byte order)
+  // NetBSD kernel delivers IDENTIFY data in host byte order
   if (isbigendian()){
     int i;
     
@@ -771,7 +795,25 @@ int ataVersionInfo (const char** description, struct ata_identify_device *drive,
       return std;
     }
   }
-  
+
+  // Try new ATA-8 minor revision numbers (Table 32 of T13/1699-D Revision 4c)
+  // (not in actual_ver/minor_str to avoid large sparse tables)
+  const char *desc;
+  switch (*minor) {
+    case 0x0027: desc = "ATA-8-ACS revision 3c"; break;
+    case 0x0029: desc = "ATA-8-ACS revision 4"; break;
+    case 0x0033: desc = "ATA-8-ACS revision 3e"; break;
+    case 0x0039: desc = "ATA-8-ACS revision 4c"; break;
+    case 0x0042: desc = "ATA-8-ACS revision 3f"; break;
+    case 0x0052: desc = "ATA-8-ACS revision 3b"; break;
+    case 0x0107: desc = "ATA-8-ACS revision 2d"; break;
+    default:     desc = 0; break;
+  }
+  if (desc) {
+    *description = desc;
+    return 8;
+  }
+
   // HDPARM has a very complicated algorithm from here on. Since SMART only
   // exists on ATA-3 and later standards, let's punt on this.  If you don't
   // like it, please fix it.  The code's in CVS.
@@ -827,7 +869,7 @@ int ataReadSmartValues(int device, struct ata_smart_values *data){
   if (checksum((unsigned char *)data))
     checksumwarning("SMART Attribute Data Structure");
   
-  // byte swap if needed
+  // swap endian order if needed
   if (isbigendian()){
     int i;
     swap2((char *)&(data->revnumber));
@@ -879,7 +921,7 @@ int ataReadSelfTestLog (int device, struct ata_smart_selftestlog *data){
   if (con->fixfirmwarebug == FIX_SAMSUNG)
     fixsamsungselftestlog(data);
 
-  // fix endian order, if needed
+  // swap endian order if needed
   if (isbigendian()){
     int i;
     swap2((char*)&(data->revnumber));
@@ -945,13 +987,17 @@ int ataReadSelectiveSelfTestLog(int device, struct ata_selective_self_test_log *
 }
 
 // Writes the selective self-test log (log #9)
-int ataWriteSelectiveSelfTestLog(int device, struct ata_smart_values *sv){   
-  int i;
-  struct ata_selective_self_test_log sstlog, *data=&sstlog;
-  unsigned char cksum=0;
-  unsigned char *ptr=(unsigned char *)data;
-  
+int ataWriteSelectiveSelfTestLog(int device, struct ata_smart_values *sv, uint64_t num_sectors){   
+
+  // Disk size must be known
+  if (!num_sectors) {
+    pout("Disk size is unknown, unable to check selective self-test spans\n");
+    return -1;
+  }
+
   // Read log
+  struct ata_selective_self_test_log sstlog, *data=&sstlog;
+  unsigned char *ptr=(unsigned char *)data;
   if (ataReadSelectiveSelfTestLog(device, data)) {
     pout("Since Read failed, will not attempt to WRITE Selective Self-test Log\n");
     return -1;
@@ -977,7 +1023,81 @@ int ataWriteSelectiveSelfTestLog(int device, struct ata_smart_values *sv){
     pout("Error SMART Selective or other Self-Test in progress.\n");
     return -4;
   }
-  
+
+  // Set start/end values based on old spans for special -t select,... options
+  int i;
+  for (i=0; i<con->smartselectivenumspans; i++) {
+    char mode = con->smartselectivemode[i];
+    uint64_t start = con->smartselectivespan[i][0];
+    uint64_t end   = con->smartselectivespan[i][1];
+    if (mode == SEL_CONT) {// redo or next dependig on last test status
+      switch (sv->self_test_exec_status >> 4) {
+        case 1: case 2: // Aborted/Interrupted by host
+          pout("Continue Selective Self-Test: Redo last span\n");
+          mode = SEL_REDO;
+          break;
+        default: // All others
+          pout("Continue Selective Self-Test: Start next span\n");
+          mode = SEL_NEXT;
+          break;
+      }
+    }
+    switch (mode) {
+      case SEL_RANGE: // -t select,START-END
+        break;
+      case SEL_REDO: // -t select,redo... => Redo current
+        start = data->span[i].start;
+        if (end > 0) { // -t select,redo+SIZE
+          end--; end += start; // [oldstart, oldstart+SIZE)
+        }
+        else // -t select,redo
+          end = data->span[i].end; // [oldstart, oldend]
+        break;
+      case SEL_NEXT: // -t select,next... => Do next
+        if (data->span[i].end == 0) {
+          start = end = 0; break; // skip empty spans
+        }
+        start = data->span[i].end + 1;
+        if (start >= num_sectors)
+          start = 0; // wrap around
+        if (end > 0) { // -t select,next+SIZE
+          end--; end += start; // (oldend, oldend+SIZE]
+        }
+        else { // -t select,next
+          uint64_t oldsize = data->span[i].end - data->span[i].start + 1;
+          end = start + oldsize - 1; // (oldend, oldend+oldsize]
+          if (end >= num_sectors) {
+            // Adjust size to allow round-robin testing without future size decrease
+            uint64_t spans = (num_sectors + oldsize-1) / oldsize;
+            uint64_t newsize = (num_sectors + spans-1) / spans;
+            uint64_t newstart = num_sectors - newsize, newend = num_sectors - 1;
+            pout("Span %d changed from %"PRIu64"-%"PRIu64" (%"PRIu64" sectors)\n"
+                 "                 to %"PRIu64"-%"PRIu64" (%"PRIu64" sectors) (%"PRIu64" spans)\n",
+                i, start, end, oldsize, newstart, newend, newsize, spans);
+            start = newstart; end = newend;
+          }
+        }
+        break;
+      default:
+        pout("ataWriteSelectiveSelfTestLog: Invalid mode %d\n", mode);
+        return -1;
+    }
+    // Range check
+    if (start < num_sectors && num_sectors <= end) {
+      if (end != ~(uint64_t)0) // -t select,N-max
+        pout("Size of self-test span %d decreased according to disk size\n", i);
+      end = num_sectors - 1;
+    }
+    if (!(start <= end && end < num_sectors)) {
+      pout("Invalid selective self-test span %d: %"PRIu64"-%"PRIu64" (%"PRIu64" sectors)\n",
+        i, start, end, num_sectors);
+      return -1;
+    }
+    // Write back to allow ataSmartTest() to print the actual values
+    con->smartselectivespan[i][0] = start;
+    con->smartselectivespan[i][1] = end;
+  }
+
   // Clear spans
   for (i=0; i<5; i++)
     memset(data->span+i, 0, sizeof(struct test_span));
@@ -1010,17 +1130,17 @@ int ataWriteSelectiveSelfTestLog(int device, struct ata_smart_values *sv){
 
   // Set checksum to zero, then compute checksum
   data->checksum=0;
+  unsigned char cksum=0;
   for (i=0; i<512; i++)
     cksum+=ptr[i];
   cksum=~cksum;
   cksum+=1;
   data->checksum=cksum;
 
-    // swap endian order if needed
+  // swap endian order if needed
   if (isbigendian()){
-    int i;
     swap2((char *)&(data->logversion));
-    for (i=0;i<5;i++){
+    for (int i=0;i<5;i++){
       swap8((char *)&(data->span[i].start));
       swap8((char *)&(data->span[i].end));
     }
@@ -1092,7 +1212,7 @@ int ataReadErrorLog (int device, struct ata_smart_errorlog *data){
   else if (con->fixfirmwarebug == FIX_SAMSUNG2)
     fixsamsungerrorlog2(data);
 
-  // Correct endian order if necessary
+  // swap endian order if needed
   if (isbigendian()){
     int i,j;
     
@@ -1125,7 +1245,7 @@ int ataReadSmartThresholds (int device, struct ata_smart_thresholds_pvt *data){
   if (checksum((unsigned char *)data))
     checksumwarning("SMART Attribute Thresholds Structure");
   
-  // byte swap if needed
+  // swap endian order if needed
   if (isbigendian())
     swap2((char *)&(data->revnumber));
 
@@ -1212,7 +1332,8 @@ int ataSmartStatus2(int device){
 
 // This is the way to execute ALL tests: offline, short self-test,
 // extended self test, with and without captive mode, etc.
-int ataSmartTest(int device, int testtype, struct ata_smart_values *sv) {     
+int ataSmartTest(int device, int testtype, struct ata_smart_values *sv, uint64_t num_sectors)
+{
   char cmdmsg[128],*type,*captive;
   int errornum, cap, retval, select=0;
 
@@ -1240,7 +1361,7 @@ int ataSmartTest(int device, int testtype, struct ata_smart_values *sv) {
   
   // If doing a selective self-test, first use WRITE_LOG to write the
   // selective self-test log.
-  if (select && (retval=ataWriteSelectiveSelfTestLog(device, sv))) {
+  if (select && (retval=ataWriteSelectiveSelfTestLog(device, sv, num_sectors))) {
     if (retval==-4)
       pout("Can't start selective self-test without aborting current test: use '-X' option to smartctl.\n");
     return retval;
@@ -1484,6 +1605,32 @@ int ataCheckAttribute(struct ata_smart_values *data,
 }
 
 
+// Print temperature value and Min/Max value if present
+static void ataPrintTemperatureValue(char *out, const unsigned char *raw, const unsigned *word)
+{
+  out+=sprintf(out, "%u", word[0]);
+  if (!word[1] && !word[2])
+    return; // No Min/Max
+
+  unsigned lo = ~0, hi = ~0;
+  if (!raw[3]) {
+    // 00 HH 00 LL 00 TT (IBM)
+    hi = word[2]; lo = word[1];
+  }
+  else if (!word[2]) {
+    // 00 00 HH LL 00 TT (Maxtor)
+    hi = raw[3]; lo = raw[2];
+  }
+  if (lo > hi) {
+    unsigned t = lo; lo = hi; hi = t;
+  }
+  if (lo <= word[0] && word[0] <= hi)
+    sprintf(out, " (Lifetime Min/Max %u/%u)", lo, hi);
+  else
+    sprintf(out, " (%u %u %u %u)", raw[5], raw[4], raw[3], raw[2]);
+}
+
+
 // This routine prints the raw value of an attribute as a text string
 // into out. It also returns this 48-bit number as a long long.  The
 // array defs[] contains non-zero values if particular attributes have
@@ -1579,6 +1726,10 @@ int64_t ataPrintSmartAttribRawValue(char *out,
       // hours
       out+=sprintf(out, "%"PRIu64, rawvalue);  //stored in hours
     break;
+    // Temperature
+  case 190:
+    ataPrintTemperatureValue(out, attribute->raw, word);
+    break;
    // Load unload cycles
   case 193:
     if (select==1){
@@ -1602,15 +1753,8 @@ int64_t ataPrintSmartAttribRawValue(char *out,
     else if (select==2)
       // unknown attribute
       out+=sprintf(out, "%"PRIu64, rawvalue);
-    else {
-      out+=sprintf(out, "%d", word[0]);
-      if (!(rawvalue==word[0])) {
-	int min=word[1]<word[2]?word[1]:word[2];
-	int max=word[1]>word[2]?word[1]:word[2];
-        // The other bytes are in use. Try IBM's model
-        out+=sprintf(out, " (Lifetime Min/Max %d/%d)", min, max);
-      }
-    }
+    else
+      ataPrintTemperatureValue(out, attribute->raw, word);
     break;
   default:
     out+=sprintf(out, "%"PRIu64, rawvalue);
@@ -1692,6 +1836,12 @@ void ataPrintSmartAttribName(char *out, unsigned char id, unsigned char *definit
   case 13:
     name="Read_Soft_Error_Rate";
     break;
+  case 187:
+    name="Reported_Uncorrect";
+    break;
+  case 189:
+    name="High_Fly_Writes";
+    break;
   case 190:
     // Western Digital uses this for temperature.
     // It's identical to Attribute 194 except that it
@@ -1700,7 +1850,7 @@ void ataPrintSmartAttribName(char *out, unsigned char id, unsigned char *definit
     // is typically 55C.  So if this attribute has failed
     // in the past, it indicates that the drive temp exceeded
     // 55C sometime in the past.
-    name="Temperature_Celsius";
+    name="Airflow_Temperature_Cel";
     break;
   case 191:
     name="G-Sense_Error_Rate";
@@ -1934,3 +2084,399 @@ unsigned char ATAReturnTemperatureValue(/*const*/ struct ata_smart_values *data,
   // No valid attribute found
   return 0;
 }
+
+// Read SCT Status
+int ataReadSCTStatus(int device, ata_sct_status_response * sts)
+{
+  // read SCT status via SMART log 0xe0
+  memset(sts, 0, sizeof(*sts));
+  if (smartcommandhandler(device, READ_LOG, 0xe0, (char *)sts)){
+    syserror("Error Read SCT Status failed");
+    return -1;
+  }
+
+  // swap endian order if needed
+  if (isbigendian()){
+    swapx(&sts->format_version);
+    swapx(&sts->sct_version);
+    swapx(&sts->sct_spec);
+    swapx(&sts->ext_status_code);
+    swapx(&sts->action_code);
+    swapx(&sts->function_code);
+    swapx(&sts->over_limit_count);
+    swapx(&sts->under_limit_count);
+  }
+
+  // Check format version
+  if (!(sts->format_version == 2 || sts->format_version == 3)) {
+    pout("Error unknown SCT Status format version %u, should be 2 or 3.\n", sts->format_version);
+    return -1;
+  }
+  return 0;
+}
+
+// Read SCT Temperature History Table and Status
+int ataReadSCTTempHist(int device, ata_sct_temperature_history_table * tmh,
+                       ata_sct_status_response * sts)
+{
+  // Check initial status
+  if (ataReadSCTStatus(device, sts))
+    return -1;
+
+  // Do nothing if other SCT command is executing
+  if (sts->ext_status_code == 0xffff) {
+    pout("Another SCT command is executing, abort Read Data Table\n"
+         "(SCT ext_status_code 0x%04x, action_code=%u, function_code=%u)\n",
+      sts->ext_status_code, sts->action_code, sts->function_code);
+    return -1;
+  }
+
+  ata_sct_data_table_command cmd; memset(&cmd, 0, sizeof(cmd));
+  // CAUTION: DO NOT CHANGE THIS VALUE (SOME ACTION CODES MAY ERASE DISK)
+  cmd.action_code   = 5; // Data table command
+  cmd.function_code = 1; // Read table
+  cmd.table_id      = 2; // Temperature History Table
+
+  // write command via SMART log page 0xe0
+  if (smartcommandhandler(device, WRITE_LOG, 0xe0, (char *)&cmd)){
+    syserror("Error Write SCT Data Table command failed");
+    return -1;
+  }
+
+  // read SCT data via SMART log page 0xe1
+  memset(tmh, 0, sizeof(*tmh));
+  if (smartcommandhandler(device, READ_LOG, 0xe1, (char *)tmh)){
+    syserror("Error Read SCT Data Table failed");
+    return -1;
+  }
+
+  // re-read and check SCT status
+  if (ataReadSCTStatus(device, sts))
+    return -1;
+
+  if (!(sts->ext_status_code == 0 && sts->action_code == 5 && sts->function_code == 1)) {
+    pout("Error unexcepted SCT status 0x%04x (action_code=%u, function_code=%u)\n",
+      sts->ext_status_code, sts->action_code, sts->function_code);
+    return -1;
+  }
+
+  // swap endian order if needed
+  if (isbigendian()){
+    swapx(&tmh->format_version);
+    swapx(&tmh->sampling_period);
+    swapx(&tmh->interval);
+  }
+
+  // Check format version
+  if (tmh->format_version != 2) {
+    pout("Error unknown SCT Temperature History Format Version (%u), should be 2.\n", tmh->format_version);
+    return -1;
+  }
+  return 0;
+}
+
+// Set SCT Temperature Logging Interval
+int ataSetSCTTempInterval(int device, unsigned interval, bool persistent)
+{
+  // Check initial status
+  ata_sct_status_response sts;
+  if (ataReadSCTStatus(device, &sts))
+    return -1;
+
+  // Do nothing if other SCT command is executing
+  if (sts.ext_status_code == 0xffff) {
+    pout("Another SCT command is executing, abort Feature Control\n"
+         "(SCT ext_status_code 0x%04x, action_code=%u, function_code=%u)\n",
+      sts.ext_status_code, sts.action_code, sts.function_code);
+    return -1;
+  }
+
+  ata_sct_feature_control_command cmd; memset(&cmd, 0, sizeof(cmd));
+  // CAUTION: DO NOT CHANGE THIS VALUE (SOME ACTION CODES MAY ERASE DISK)
+  cmd.action_code   = 4; // Feature Control command
+  cmd.function_code = 1; // Set state
+  cmd.feature_code  = 3; // Temperature logging interval
+  cmd.state         = interval;
+  cmd.option_flags  = (persistent ? 0x01 : 0x00);
+
+  // write command via SMART log page 0xe0
+  if (smartcommandhandler(device, WRITE_LOG, 0xe0, (char *)&cmd)){
+    syserror("Error Write SCT Feature Control Command failed");
+    return -1;
+  }
+
+  // re-read and check SCT status
+  if (ataReadSCTStatus(device, &sts))
+    return -1;
+
+  if (!(sts.ext_status_code == 0 && sts.action_code == 4 && sts.function_code == 1)) {
+    pout("Error unexcepted SCT status 0x%04x (action_code=%u, function_code=%u)\n",
+      sts.ext_status_code, sts.action_code, sts.function_code);
+    return -1;
+  }
+  return 0;
+}
+
+
+/////////////////////////////////////////////////////////////////////////////
+// Pseudo-device to parse "smartctl -r ataioctl,2 ..." output and simulate
+// an ATA device with same behaviour
+
+// Table of parsed commands, return value, data
+struct parsed_ata_command
+{ 
+  smart_command_set command;
+  int select;
+  int retval, errval;
+  char * data;
+};
+
+const int max_num_parsed_commands = 32;
+static parsed_ata_command parsed_command_table[max_num_parsed_commands];
+static int num_parsed_commands;
+static int next_replay_command;
+static bool replay_out_of_sync;
+
+
+static const char * nextline(const char * s, int & lineno)
+{
+  for (s += strcspn(s, "\r\n"); *s == '\r' || *s == '\n'; s++) {
+    if (*s == '\r' && s[1] == '\n')
+      s++;
+    lineno++;
+  }
+  return s;
+}
+
+static int name2command(const char * s)
+{
+  for (int i = 0; i < (int)(sizeof(commandstrings)/sizeof(commandstrings[0])); i++) {
+    if (!strcmp(s, commandstrings[i]))
+      return i;
+  }
+  return -1;
+}
+
+static bool matchcpy(char * dest, size_t size, const char * src, const regmatch_t & srcmatch)
+{
+  if (srcmatch.rm_so < 0)
+    return false;
+  size_t n = srcmatch.rm_eo - srcmatch.rm_so;
+  if (n >= size)
+    n = size-1;
+  memcpy(dest, src + srcmatch.rm_so, n);
+  dest[n] = 0;
+  return true;
+}
+
+static inline int matchtoi(const char * src, const regmatch_t & srcmatch, int defval)
+{
+  if (srcmatch.rm_so < 0)
+    return defval;
+  return atoi(src + srcmatch.rm_so);
+}
+
+
+// Parse stdin and build command table
+int parsedev_open(const char * pathname)
+{
+  if (strcmp(pathname, "-")) {
+    errno = EINVAL; return -1;
+  }
+  pathname = "<stdin>";
+  // Fill buffer
+  char buffer[64*1024];
+  int size = 0;
+  while (size < (int)sizeof(buffer)) {
+    int nr = fread(buffer, 1, sizeof(buffer), stdin);
+    if (nr <= 0)
+      break;
+    size += nr;
+  }
+  if (size <= 0) {
+    pout("%s: Unexpected EOF\n", pathname);
+    errno = ENOENT; return -1;
+  }
+  if (size >= (int)sizeof(buffer)) {
+    pout("%s: Buffer overflow\n", pathname);
+    errno = EIO; return -1;
+  }
+  buffer[size] = 0;
+
+  // Regex to match output from "-r ataioctl,2"
+  static const char pattern[] = "^"
+  "(" // (1
+    "REPORT-IOCTL: DeviceFD=[0-9]+ Command=([A-Z ]*[A-Z])" // (2)
+    "(" // (3
+      "( InputParameter=([0-9]+))?" // (4 (5))
+    "|"
+      "( returned (-?[0-9]+)( errno=([0-9]+)[^\r\n]*)?)" // (6 (7) (8 (9)))
+    ")" // )
+    "[\r\n]" // EOL match necessary to match optional parts above
+  "|"
+    "===== \\[([A-Z ]*[A-Z])\\] DATA START " // (10)
+  ")"; // )
+
+  // Compile regex
+  regex_t rex;
+  if (compileregex(&rex, pattern, REG_EXTENDED)) {
+    errno = EIO; return -1;
+  }
+
+  // Parse buffer
+  const char * errmsg = 0;
+  int i = -1, state = 0, lineno = 1;
+  for (const char * line = buffer; *line; line = nextline(line, lineno)) {
+    // Match line
+    if (!(line[0] == 'R' || line[0] == '='))
+      continue;
+    const int nmatch = 1+10;
+    regmatch_t match[nmatch];
+    if (regexec(&rex, line, nmatch, match, 0))
+      continue;
+
+    char cmdname[40];
+    if (matchcpy(cmdname, sizeof(cmdname), line, match[2])) { // "REPORT-IOCTL:... Command=%s ..."
+      int nc = name2command(cmdname);
+      if (nc < 0) {
+        errmsg = "Unknown ATA command name"; break;
+      }
+      if (match[7].rm_so < 0) { // "returned %d"
+        // Start of command
+        if (!(state == 0 || state == 2)) {
+          errmsg = "Missing REPORT-IOCTL result"; break;
+        }
+        if (++i >= max_num_parsed_commands) {
+          errmsg = "Too many ATA commands"; break;
+        }
+        parsed_command_table[i].command = (smart_command_set)nc;
+        parsed_command_table[i].select = matchtoi(line, match[5], 0); // "InputParameter=%d"
+        state = 1;
+      }
+      else {
+        // End of command
+        if (!(state == 1 && (int)parsed_command_table[i].command == nc)) {
+          errmsg = "Missing REPORT-IOCTL start"; break;
+        }
+        parsed_command_table[i].retval = matchtoi(line, match[7], -1); // "returned %d"
+        parsed_command_table[i].errval = matchtoi(line, match[9], 0); // "errno=%d"
+        state = 2;
+      }
+    }
+    else if (matchcpy(cmdname, sizeof(cmdname), line, match[10])) { // "===== [%s] DATA START "
+      // Start of sector hexdump
+      int nc = name2command(cmdname);
+      if (!(state == (nc == WRITE_LOG ? 1 : 2) && (int)parsed_command_table[i].command == nc)) {
+          errmsg = "Unexpected DATA START"; break;
+      }
+      line = nextline(line, lineno);
+      char * data = (char *)malloc(512);
+      unsigned j;
+      for (j = 0; j < 32; j++) {
+        unsigned b[16];
+        unsigned u1, u2; int n1 = -1;
+        if (!(sscanf(line, "%3u-%3u: "
+                        "%2x %2x %2x %2x %2x %2x %2x %2x "
+                        "%2x %2x %2x %2x %2x %2x %2x %2x%n",
+                     &u1, &u2,
+                     b+ 0, b+ 1, b+ 2, b+ 3, b+ 4, b+ 5, b+ 6, b+ 7,
+                     b+ 8, b+ 9, b+10, b+11, b+12, b+13, b+14, b+15, &n1) == 18
+              && n1 >= 56 && u1 == j*16 && u2 == j*16+15))
+          break;
+        for (unsigned k = 0; k < 16; k++)
+          data[j*16+k] = b[k];
+        line = nextline(line, lineno);
+      }
+      if (j < 32) {
+        free(data);
+        errmsg = "Incomplete sector hex dump"; break;
+      }
+      parsed_command_table[i].data = data;
+      if (nc != WRITE_LOG)
+        state = 0;
+    }
+  }
+
+  if (!(state == 0 || state == 2))
+    errmsg = "Missing REPORT-IOCTL result";
+
+  if (!errmsg && i < 0)
+    errmsg = "No information found";
+
+  num_parsed_commands = i+1;
+  next_replay_command = 0;
+  replay_out_of_sync = false;
+
+  if (errmsg) {
+    pout("%s(%d): Syntax error: %s\n", pathname, lineno, errmsg);
+    errno = EIO;
+    parsedev_close(0);
+    return -1;
+  }
+  return 0;
+}
+
+// Report warnings and free command table 
+void parsedev_close(int /*fd*/)
+{
+  if (replay_out_of_sync)
+      pout("REPLAY-IOCTL: Warning: commands replayed out of sync\n");
+  else if (next_replay_command != 0)
+      pout("REPLAY-IOCTL: Warning: %d command(s) not replayed\n", num_parsed_commands-next_replay_command);
+
+  for (int i = 0; i < num_parsed_commands; i++) {
+    if (parsed_command_table[i].data) {
+      free(parsed_command_table[i].data); parsed_command_table[i].data = 0;
+    }
+  }
+  num_parsed_commands = 0;
+}
+
+// Simulate ATA command from command table
+static int parsedev_command_interface(int /*fd*/, smart_command_set command, int select, char * data)
+{
+  // Find command, try round-robin of out of sync
+  int i = next_replay_command;
+  for (int j = 0; ; j++) {
+    if (j >= num_parsed_commands) {
+      pout("REPLAY-IOCTL: Warning: Command not found\n");
+      errno = ENOSYS;
+      return -1;
+    }
+    if (parsed_command_table[i].command == command && parsed_command_table[i].select == select)
+      break;
+    if (!replay_out_of_sync) {
+      replay_out_of_sync = true;
+      pout("REPLAY-IOCTL: Warning: Command #%d is out of sync\n", i+1);
+    }
+    if (++i >= num_parsed_commands)
+      i = 0;
+  }
+  next_replay_command = i;
+  if (++next_replay_command >= num_parsed_commands)
+    next_replay_command = 0;
+
+  // Return command data
+  switch (command) {
+    case IDENTIFY:
+    case PIDENTIFY:
+    case READ_VALUES:
+    case READ_THRESHOLDS:
+    case READ_LOG:
+      if (parsed_command_table[i].data)
+        memcpy(data, parsed_command_table[i].data, 512);
+      break;
+    case WRITE_LOG:
+      if (!(parsed_command_table[i].data && !memcmp(data, parsed_command_table[i].data, 512)))
+        pout("REPLAY-IOCTL: Warning: WRITE LOG data does not match\n");
+      break;
+    case CHECK_POWER_MODE:
+      data[0] = (char)0xff;
+    default:
+      break;
+  }
+
+  if (parsed_command_table[i].errval)
+    errno = parsed_command_table[i].errval;
+  return parsed_command_table[i].retval;
+}
