@@ -4,7 +4,7 @@
  * Home page of code is: http://smartmontools.sourceforge.net
  * Address of support mailing list: smartmontools-support@lists.sourceforge.net
  *
- * Copyright (C) 2003-6 Philip Williams, Bruce Allen
+ * Copyright (C) 2003-8 Philip Williams, Bruce Allen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include "knowndrives.h"
 #include "utility.h" // includes <regex.h>
 
-const char *knowndrives_c_cvsid="$Id: knowndrives.cpp,v 1.156 2006/10/30 22:35:28 pjwilliams Exp $"
+const char *knowndrives_c_cvsid="$Id: knowndrives.cpp,v 1.166 2008/02/02 18:10:48 chrfranke Exp $"
 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
 
 #define MODEL_STRING_LENGTH                         40
@@ -111,11 +111,13 @@ const unsigned char vendoropts_Hitachi_DK23XX[][2] = {
 
 const char same_as_minus_F[]="Fixes byte order in some SMART data (same as -F samsung)";
 const char same_as_minus_F2[]="Fixes byte order in some SMART data (same as -F samsung2)";
+const char same_as_minus_F3[]="Fixes completed self-test reported as in progress (same as -F samsung3)";
 
 const char may_need_minus_F_disabled[] ="May need -F samsung disabled; see manual for details.";
 const char may_need_minus_F2_disabled[]="May need -F samsung2 disabled; see manual for details.";
 const char may_need_minus_F2_enabled[] ="May need -F samsung2 enabled; see manual for details.";
 const char may_need_minus_F_enabled[]  ="May need -F samsung or -F samsung2 enabled; see manual for details.";
+const char may_need_minus_F3_enabled[] ="May need -F samsung3 enabled; see manual for details.";
 
 /* Special-purpose functions for use in knowndrives[]. */
 void specialpurpose_reverse_samsung(smartmonctrl *con)
@@ -128,6 +130,11 @@ void specialpurpose_reverse_samsung2(smartmonctrl *con)
   if (con->fixfirmwarebug==FIX_NOTSPECIFIED)
     con->fixfirmwarebug = FIX_SAMSUNG2;
 }
+void specialpurpose_fix_samsung3(smartmonctrl *con)
+{
+  if (con->fixfirmwarebug==FIX_NOTSPECIFIED)
+    con->fixfirmwarebug = FIX_SAMSUNG3;
+}
 
 /* Table of settings for known drives terminated by an element containing all
  * zeros.  The drivesettings structure is described in knowndrives.h.  Note
@@ -280,6 +287,11 @@ const drivesettings knowndrives[] = {
     vendoropts_9_seconds,
     NULL, NULL
   },
+  { "Fujitsu MHW2 BH",
+    "^FUJITSU MHW2(04|06|08|10|12|16)0BH$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { NULL, // Samsung SV4012H (known firmware)
     "^SAMSUNG SV4012H$",
     "^RM100-08$",
@@ -357,19 +369,45 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { "SAMSUNG SpinPoint P120 series", // tested with SP2504C/VT100-33
-    "^SAMSUNG SP(16[01]3|2[05][01]4)[CN]$",
+  { "SAMSUNG SpinPoint T166 series", // tested with HD501LJ/CR100-10
+    "^SAMSUNG HD(080G|160H|32[01]K|403L|50[01]L)J$",
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "SAMSUNG SpinPoint P120 series", // VF100-37 firmware, tested with SP2514N/VF100-37
+    "^SAMSUNG SP(16[01]3|2[05][01]4)[CN]$",
+    "^VF100-37$",
+    NULL, NULL,
+    specialpurpose_fix_samsung3,
+    same_as_minus_F3
+  },
+  { "SAMSUNG SpinPoint P120 series", // other firmware, tested with SP2504C/VT100-33
+    "^SAMSUNG SP(16[01]3|2[05][01]4)[CN]$",
+    ".*",
+    may_need_minus_F3_enabled,
+    NULL, NULL, NULL
+  },
   { "SAMSUNG SpinPoint P80 SD series", // tested with HD160JJ/ZM100-33
     "^SAMSUNG HD(080H|120I|160J)J$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { "SAMSUNG SpinPoint P80 series", // firmware *-26 or later, tested with SP1614C/SW100-34
+  { "SAMSUNG SpinPoint P80 series", // BH100-35 firmware, tested with SP0842N/BH100-35
     "^SAMSUNG SP(0451|08[0124]2|12[0145]3|16[0145]4)[CN]$",
-    ".*-(2[6789]|3[0-9])$",
+    "^BH100-35$",
+    NULL, NULL,
+    specialpurpose_fix_samsung3,
+    same_as_minus_F3
+  },
+  { "SAMSUNG SpinPoint P80 series", // firmware *-35 or later
+    "^SAMSUNG SP(0451|08[0124]2|12[0145]3|16[0145]4)[CN]$",
+    ".*-3[5-9]$",
+    may_need_minus_F3_enabled,
+    NULL, NULL, NULL
+  },
+  { "SAMSUNG SpinPoint P80 series", // firmware *-25...34, tested with SP1614C/SW100-25 and -34
+    "^SAMSUNG SP(0451|08[0124]2|12[0145]3|16[0145]4)[CN]$",
+    ".*-(2[5-9]|3[0-4])$",
     NULL,
     vendoropts_9_halfminutes,
     NULL, NULL
@@ -617,6 +655,21 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Maxtor DiamondMax 17",
+    "^Maxtor 6G(080L|160[PE])0$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Seagate Maxtor DiamondMax 20",
+    "^MAXTOR STM3(40|80|160)[28]1[12]0?AS?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Seagate Maxtor DiamondMax 21",
+    "^MAXTOR STM3(250|320)820AS?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { "Maxtor MaXLine Plus II",
     "^Maxtor 7Y250[PM]0$",
     ".*",
@@ -724,38 +777,58 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Hitachi Travelstar 4K40",
+    "^(Hitachi )?HTS4240[234]0M9AT00$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
   { "Hitachi Travelstar 5K80 family",
-    "^HTS5480[8642]0M9AT00$",
+    "^(Hitachi )?HTS5480[8642]0M9AT00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
   { "Hitachi Travelstar 5K100 series",
-    "^HTS5410[1864]0G9(AT|SA)00$",
+    "^(Hitachi )?HTS5410[1864]0G9(AT|SA)00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
   { "Hitachi Travelstar E5K100 series",
-    "^HTE541040G9(AT|SA)00$",
+    "^(Hitachi )?HTE541040G9(AT|SA)00$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { "Hitachi Travelstar 5K120",
+    "^(Hitachi )?HTS5412(60|80|10|12)H9(AT|SA)00$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { "Hitachi Travelstar 5K160 series",
+    "^(Hitachi )?HTS5416([468]0|1[26])J9(AT|SA)00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
   { "Hitachi Travelstar 7K60",
-    "^HTS726060M9AT00$",
+    "^(Hitachi )?HTS726060M9AT00$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { "Hitachi Travelstar E7K60",
+    "^(Hitachi )?HTE7260[46]0M9AT00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
   { "Hitachi Travelstar 7K100",
-    "^HTS7210[168]0G9(AT|SA)00$",
+    "^(Hitachi )?HTS7210[168]0G9(AT|SA)00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
   { "Hitachi Travelstar E7K100",
-    "^HTE7210[168]0G9(AT|SA)00$",
+    "^(Hitachi )?HTE7210[168]0G9(AT|SA)00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
-  { "Hitachi Travelstar E7K60 family",
-    "^HTE7260[46]0M9AT00$",
+  { "Hitachi Travelstar 7K200",
+    "^(Hitachi )?HTS7220(80|10|12|16|20)K9(A3|SA)00$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
@@ -784,6 +857,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Hitachi Deskstar 7K160",
+    "^(Hitachi )?HDS7216(80|16)PLA[3T]80$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { "Hitachi Deskstar 7K250 series",
     "^(Hitachi )?HDS7225((40|80|12|16)VLAT20|(12|16|25)VLAT80|(80|12|16|25)VLSA80)$",
     ".*",
@@ -804,8 +882,23 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Hitachi Deskstar T7K500",
+    "^(Hitachi )?HDT7250(25|32|40|50)VLA(360|380|T80)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Hitachi Deskstar 7K1000",
+    "^(Hitachi )?HDS7210(75|10)KLA330$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Hitachi Ultrastar 7K1000",
+    "^(Hitachi )?HUA7210(50|75|10)KLA330$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { "Toshiba 2.5\" HDD series (30-60 GB)",
-    "^TOSHIBA MK((6034|4032)GSX|(6034|4032)GAX|(6026|4026|4019|3019)GAXB?|(6025|6021|4025|4021|4018|3021|3018)GAS|(4036|3029)GACE?|(4018|3017)GAP)$",
+    "^TOSHIBA MK((6034|4032)GSX|(6034|4032)GAX|(6026|4026|4019|3019)GAXB?|(6025|6021|4025|4021|4018|3025|3021|3018)GAS|(4036|3029)GACE?|(4018|3017)GAP)$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -814,6 +907,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Toshiba 1.8\" HDD series",
+    "^TOSHIBA MK[23468]00[4-9]GA[HL]$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { NULL, // TOSHIBA MK6022GAX
     "^TOSHIBA MK6022GAX$",
     ".*",
@@ -854,11 +952,26 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Seagate Momentus 5400.3",
+    "^ST9(4081[45]|6081[35]|8081[15]|100828|120822|160821)AS?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Seagate Momentus 5400.3 ED",
+    "^ST9(4081[45]|6081[35]|8081[15]|100828|120822|160821)AB$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { "Seagate Momentus 7200.1 series",
     "^ST9(10021|80825|6023|4015)AS?$", 
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { "Seagate Momentus 7200.2",
+    "^ST9(80813|100821|120823|160823|200420)ASG?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { "Seagate Medalist 2110, 3221, 4321, 6531, and 8641",
     "^ST3(2110|3221|4321|6531|8641)A$",
     ".*",
@@ -935,12 +1048,22 @@ const drivesettings knowndrives[] = {
     NULL, NULL, NULL, NULL
   },
   { "Seagate Barracuda 7200.9 family",
-    "^ST3(500[68]41|400[68]33|300[68]22|250[68]24|250[68]24|200827|160[28]12|120814|120[28]13|80[28]110|402111)AS?$",
+    "^ST3(402111?|80[28]110?|120[28]1[0134]|160[28]1[012]|200827|250[68]24|300[68]22|(320|400)[68]33|500[68](32|41))AS?$",
     ".*",
     NULL, NULL, NULL, NULL
   },
   { "Seagate Barracuda 7200.10 family",
-    "^ST3(750[68]4|500[68]3|400[68]2|320[68]2|300[68]2|250[68]2|20082)0AS?$",
+    "^ST3((80|160)[28]15|200820|250[34]10|(250|300|320|400)[68]20|500[68]30|750[68]40)AS?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Seagate Barracuda 7200.11",
+    "^ST3(500[368]2|750[36]3|1000[36]4)0AS?$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Seagate Barracuda ES",
+    "^ST3(250[68]2|32062|40062|50063|75064)0NS$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -1025,23 +1148,29 @@ const drivesettings knowndrives[] = {
    * a  non-standard manner.  These entries may need to be updated when it
    * is understood exactly how Attribute 9 should be interpreted.
    * UPDATE: this is probably explained by the WD firmware bug described in the
-   * smartmontools FAQ */
-    "^WDC WD((4|6|8|10|12|16|18|20|25|30|32)00JB|(12|20|25)00PB)-.*$",
+   * smartmontools FAQ 
+   * UPDATE 2: this does not apply to more recent models, at least WD3200AAJB */
+    "^WDC WD((4|6|8|10|12|16|18|20|25|30|32|40|50)00(JB|PB|AAJB|AAKB))-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { "Western Digital Caviar SE (Serial ATA) family",
-    "^WDC WD((4|8|12|16|20|25|32)00JD|(12|16|20|25|30|32)00JS|1600AAJS)-.*$",
+  { "Western Digital Caviar SE Serial ATA family",
+    "^WDC WD((4|8|12|16|20|25|32|40)00(JD|KD))-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { "Western Digital Caviar SE16 family",
-    "^WDC WD((25|32|40|50)00KS|4000KD)-.*$",
+  { "Western Digital Caviar Second Generation Serial ATA family",
+    "^WDC WD((8|12|16|20|25|30|32|40|50|75)00(JS|KS|AABS|AAJS|AAKS))-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { "Western Digital Caviar RE Serial ATA series",
-    "^WDC WD((12|16|25|32)00SD|2500YD|4000Y[RS]|5000YS)-.*$",
+  { "Western Digital Caviar RE Serial ATA family",
+    "^WDC WD((12|16|25|32|40|50|75)00(SD|YD|YR|YS|ABYS|AYYS))-.*$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { "Western Digital Caviar RE EIDE family",
+    "^WDC WD((12|16|25|32)00SB)-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -1051,7 +1180,7 @@ const drivesettings knowndrives[] = {
     NULL, NULL, NULL, NULL
   },
   { "Western Digital Scorpio family",
-    "^WDC WD((12|10|8|6|4)00(UE|VE|BEAS|BEVS))-.*$",
+    "^WDC WD((4|6|8|10|12|16|20|25)00(UE|VE|BEAS|BEVE|BEVS))-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -1308,8 +1437,8 @@ void showpresets(const struct ata_identify_device *drive){
   char model[MODEL_STRING_LENGTH+1], firmware[FIRMWARE_STRING_LENGTH+1];
 
   // get the drive's model/firmware strings
-  formatdriveidstring(model, (char *)drive->model, MODEL_STRING_LENGTH);
-  formatdriveidstring(firmware, (char *)drive->fw_rev, FIRMWARE_STRING_LENGTH);
+  format_ata_string(model, (char *)drive->model, MODEL_STRING_LENGTH);
+  format_ata_string(firmware, (char *)drive->fw_rev, FIRMWARE_STRING_LENGTH);
   
   // and search to see if they match values in the table
   if ((i = lookupdrive(model, firmware)) < 0) {
@@ -1355,8 +1484,8 @@ int applypresets(const struct ata_identify_device *drive, unsigned char **optspt
   opts=*optsptr;
   
   // get the drive's model/firmware strings
-  formatdriveidstring(model, (char *)drive->model, MODEL_STRING_LENGTH);
-  formatdriveidstring(firmware, (char *)drive->fw_rev, FIRMWARE_STRING_LENGTH);
+  format_ata_string(model, (char *)drive->model, MODEL_STRING_LENGTH);
+  format_ata_string(firmware, (char *)drive->fw_rev, FIRMWARE_STRING_LENGTH);
   
   // Look up the drive in knowndrives[].
   if ((i = lookupdrive(model, firmware)) >= 0) {
