@@ -3,11 +3,11 @@
  *
  * Home page of code is: http://smartmontools.sourceforge.net
  *
- * Copyright (C) 2002-6 Bruce Allen <smartmontools-support@lists.sourceforge.net>
+ * Copyright (C) 2002-8 Bruce Allen <smartmontools-support@lists.sourceforge.net>
  * Copyright (C) 2000 Michael Cornwell <cornwell@acm.org>
  *
  * Additional SCSI work:
- * Copyright (C) 2003-6 Douglas Gilbert <dougg@torque.net>
+ * Copyright (C) 2003-8 Douglas Gilbert <dougg@torque.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #ifndef SCSICMDS_H_
 #define SCSICMDS_H_
 
-#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.63 2006/11/12 04:46:32 dpgilbert Exp $\n"
+#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.66 2008/03/04 22:09:47 ballen4705 Exp $\n"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -85,7 +85,7 @@
 #endif
 
 typedef unsigned char UINT8;
-typedef char INT8;
+typedef signed char INT8;
 typedef unsigned int UINT32;
 typedef int INT32;
 
@@ -230,6 +230,7 @@ Documentation, see http://www.storage.ibm.com/techsup/hddtech/prodspecs.htm */
 #define SCSI_SK_HARDWARE_ERROR          0x4
 #define SCSI_SK_ILLEGAL_REQUEST         0x5
 #define SCSI_SK_UNIT_ATTENTION          0x6
+#define SCSI_SK_ABORTED_COMMAND         0xb
 
 /* defines for useful Additional Sense Codes (ASCs) */
 #define SCSI_ASC_NOT_READY              0x4     /* more info in ASCQ code */
@@ -253,6 +254,8 @@ Documentation, see http://www.storage.ibm.com/techsup/hddtech/prodspecs.htm */
 #define SIMPLE_ERR_BECOMING_READY       7       /* device will be ready soon */
 #define SIMPLE_ERR_TRY_AGAIN            8       /* some warning, try again */
 #define SIMPLE_ERR_MEDIUM_HARDWARE      9       /* medium or hardware error */
+#define SIMPLE_ERR_UNKNOWN              10      /* unknown sense value */
+#define SIMPLE_ERR_ABORTED_COMMAND      11      /* most likely transport error */
 
 
 /* defines for functioncode parameter in SENDDIAGNOSTIC function */
